@@ -62,6 +62,10 @@ Float Distribution1D::DiscretePDF(int index) const {
   return(func[index] / (funcInt * Count()));
 }
 
+size_t Distribution1D::GetSize()  {
+  return(sizeof(*this) + sizeof(Float)*func.size() + sizeof(Float)*cdf.size());
+}
+
 
 
 vec2f Distribution2D::SampleContinuous(const vec2f &u, Float *pdf) const {
@@ -84,7 +88,7 @@ Distribution2D::Distribution2D(const Float *func, int nu, int nv) {
   pConditionalV.reserve(nv);
   for (int v = 0; v < nv; ++v) {
     // Compute conditional sampling distribution for $\tilde{v}$
-    pConditionalV.emplace_back(new Distribution1D(&func[v * nu], nu));
+    pConditionalV.emplace_back(std::unique_ptr<Distribution1D>(new Distribution1D(&func[v * nu], nu)));
   }
   // Compute marginal sampling distribution $p[\tilde{v}]$
   std::vector<Float> marginalFunc;
@@ -93,3 +97,12 @@ Distribution2D::Distribution2D(const Float *func, int nu, int nv) {
     marginalFunc.push_back(pConditionalV[v]->funcInt);
   pMarginal.reset(new Distribution1D(&marginalFunc[0], nv));
 }
+
+size_t Distribution2D::GetSize()  {
+  size_t total_size = 0;
+  for(int i = 0; i < pConditionalV.size(); i++) {
+    total_size += pConditionalV[i]->GetSize();
+  }
+  total_size += pMarginal->GetSize();
+  return(sizeof(*this) + total_size);
+}
